@@ -21,10 +21,12 @@
 
 #include "TWUtils.h"
 #include "TWApp.h"
-#include "TeXDocument.h"
-#include "PDFDocument.h"
+#include "TeXDocumentWindow.h"
+#include "PDFDocumentWindow.h"
 #include "TWVersion.h"
 #include "GitRev.h"
+#include "Settings.h"
+#include "utils/FileVersionDatabase.h"
 
 #include <QFileDialog>
 #include <QString>
@@ -39,14 +41,8 @@
 #include <QFile>
 #include <QDirIterator>
 #include <QSignalMapper>
-#include <QCryptographicHash>
-#include <QTextStream>
 #include <QDateTime>
 
-#include <hunspell.h>
-
-#pragma mark === TWUtils ===
-
 #if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)
 // compile-time default paths - customize by defining in the .pro file
 #ifndef TW_DICPATH
@@ -101,8 +97,7 @@ const QString TWUtils::getLibraryPath(const QString& subdir, const bool updateOn
 				libPath = dicPath;
 			return libPath; // don't try to create/update the system dicts directory
 		}
-		else
-			libRootPath = QDir::homePath() + QLatin1String("/." TEXWORKS_NAME "/");
+		libRootPath = QDir::homePath() + QLatin1String("/." TEXWORKS_NAME "/");
 #else // defined(Q_OS_WIN)
 		libRootPath = QDir::homePath() + QLatin1String("/" TEXWORKS_NAME "/");
 #endif
@@ -114,6 +109,11 @@ const QString TWUtils::getLibraryPath(const QString& subdir, const bool updateOn
 	return libPath;
 }
 
+const QStringList TWUtils::getLibraryPaths(const QString & subdir, const bool updateOnDisk)
+{
+	return getLibraryPath(subdir, updateOnDisk).split(QStringLiteral(PATH_LIST_SEP));
+}
+
 /*static*/
 void TWUtils::updateLibraryResources(const QDir& srcRootDir, const QDir& destRootDir, const QString& subdir)
 {
@@ -132,7 +132,7 @@ void TWUtils::updateLibraryResources(const QDir& srcRootDir, const QDir& destRoo
 	if (subdir == QString::fromLatin1("translations")) // don't copy the built-in translations
 		return;
 	
-	FileVersionDatabase fvdb = FileVersionDatabase::load(destRootDir.absoluteFilePath(QString::fromLatin1("TwFileVersions.db")));
+	Tw::Utils::FileVersionDatabase fvdb = Tw::Utils::FileVersionDatabase::load(destRootDir.absoluteFilePath(QString::fromLatin1("TwFileVersions.db")));
 	
 	QDirIterator iter(srcDir, QDirIterator::Subdirectories);
 	while (iter.hasNext()) {
@@ -147,15 +147,15 @@ void TWUtils::updateLibraryResources(const QDir& srcRootDir, const QDir& destRoo
 
 		// Check if the file is in the database
 		if (fvdb.hasFileRecord(destPath)) {
-			FileVersionDatabase::Record rec = fvdb.getFileRecord(destPath);
+			Tw::Utils::FileVersionDatabase::Record rec = fvdb.getFileRecord(destPath);
 			// If the file no longer exists on the disk, the user has deleted it
 			// Hence we won't recreate it, but we keep the database record to
 			// remember that this file was deleted by the user
 			if (!QFileInfo(destPath).exists())
 				continue;
 			
-			QByteArray srcHash = FileVersionDatabase::hashForFile(srcPath);
-			QByteArray destHash = FileVersionDatabase::hashForFile(destPath);
+			QByteArray srcHash = Tw::Utils::FileVersionDatabase::hashForFile(srcPath);
+			QByteArray destHash = Tw::Utils::FileVersionDatabase::hashForFile(destPath);
 			// If the file was modified, don't do anything, either
 			if (destHash != rec.hash) {
 				// The only exception is if the file on the disk matches the
@@ -182,7 +182,7 @@ void TWUtils::updateLibraryResources(const QDir& srcRootDir, const QDir& destRoo
 			}
 		}
 		else {
-			QByteArray srcHash = FileVersionDatabase::hashForFile(srcPath);
+			QByteArray srcHash = Tw::Utils::FileVersionDatabase::hashForFile(srcPath);
 			// If the file is not in the database, we add it - unless a file
 			// with the name already exists
 			if (!QFileInfo(destPath).exists()) {
@@ -197,7 +197,7 @@ void TWUtils::updateLibraryResources(const QDir& srcRootDir, const QDir& destRoo
 				// If it happens to be identical with the version we would install
 				// we do take ownership, however, and register it in the
 				// database so that future updates are applied
-				QByteArray destHash = FileVersionDatabase::hashForFile(destPath);
+				QByteArray destHash = Tw::Utils::FileVersionDatabase::hashForFile(destPath);
 				if (srcHash == destHash)
 					fvdb.addFileRecord(destPath, destHash, gitCommitHash());
 			}
@@ -206,9 +206,9 @@ void TWUtils::updateLibraryResources(const QDir& srcRootDir, const QDir& destRoo
 
 	// Now, remove all files that are unmodified on disk and were
 	// removed upstream
-	QMutableListIterator<FileVersionDatabase::Record> recIt(fvdb.getFileRecords());
+	QMutableListIterator<Tw::Utils::FileVersionDatabase::Record> recIt(fvdb.getFileRecords());
 	while (recIt.hasNext()) {
-		const FileVersionDatabase::Record & rec = recIt.next();
+		const Tw::Utils::FileVersionDatabase::Record & rec = recIt.next();
 
 		QString destPath = rec.filePath.filePath();
 		QString path = destRootDir.relativeFilePath(destPath);
@@ -220,7 +220,7 @@ void TWUtils::updateLibraryResources(const QDir& srcRootDir, const QDir& destRoo
 		
 		// If the source file no longer exists but the file on disk is up to
 		// date, remove it
-		if (rec.filePath.exists() && FileVersionDatabase::hashForFile(destPath) == rec.hash) {
+		if (rec.filePath.exists() && Tw::Utils::FileVersionDatabase::hashForFile(destPath) == rec.hash) {
 			QFile(destPath).remove();
 			recIt.remove();
 		}
@@ -245,7 +245,7 @@ insertItemIfPresent(QFileInfo& fi, QMenu* helpMenu, QAction* before, QSignalMapp
 		}
 		QAction* action = new QAction(title, helpMenu);
 		mapper->setMapping(action, fi.canonicalFilePath());
-		action->connect(action, SIGNAL(triggered()), mapper, SLOT(map()));
+		QObject::connect(action, SIGNAL(triggered()), mapper, SLOT(map()));
 		helpMenu->insertAction(before, action);
 		return 1;
 	}
@@ -255,9 +255,9 @@ insertItemIfPresent(QFileInfo& fi, QMenu* helpMenu, QAction* before, QSignalMapp
 void TWUtils::insertHelpMenuItems(QMenu* helpMenu)
 {
 	QSignalMapper* mapper = new QSignalMapper(helpMenu);
-	mapper->connect(mapper, SIGNAL(mapped(const QString&)), TWApp::instance(), SLOT(openHelpFile(const QString&)));
+	QObject::connect(mapper, SIGNAL(mapped(const QString&)), TWApp::instance(), SLOT(openHelpFile(const QString&)));
 
-	QAction* before = NULL;
+	QAction* before = nullptr;
 	int i, firstSeparator = 0;
 	QList<QAction*> actions = helpMenu->actions();
 	for (i = 0; i < actions.count(); ++i) {
@@ -286,7 +286,7 @@ void TWUtils::insertHelpMenuItems(QMenu* helpMenu)
 	if (!helpPath.isEmpty())
 		helpDir.cd(QString(helpPath));
 
-	QSETTINGS_OBJECT(settings);
+	Tw::Settings settings;
 	QString loc = settings.value(QString::fromLatin1("locale")).toString();
 	if (loc.isEmpty())
 		loc = QLocale::system().name();
@@ -328,26 +328,27 @@ void TWUtils::insertHelpMenuItems(QMenu* helpMenu)
 	}
 }
 
-QList<QTextCodec*> *TWUtils::codecList = NULL;
+QList<QTextCodec*> *TWUtils::codecList = nullptr;
 
 QList<QTextCodec*> *TWUtils::findCodecs()
 {
-	if (codecList != NULL)
+	if (codecList)
 		return codecList;
 
 	codecList = new QList<QTextCodec*>;
 	QMap<QString, QTextCodec*> codecMap;
-	QRegExp iso8859RegExp(QString::fromLatin1("ISO[- ]8859-([0-9]+).*"));
+	QRegularExpression iso8859RegExp(QStringLiteral("^ISO[- ]8859-([0-9]+)"));
 	foreach (int mib, QTextCodec::availableMibs()) {
 		QTextCodec *codec = QTextCodec::codecForMib(mib);
 		QString sortKey = QString::fromUtf8(codec->name().constData()).toUpper();
+		QRegularExpressionMatch iso8859Match = iso8859RegExp.match(sortKey);
 		int rank;
 		if (sortKey.startsWith(QLatin1String("UTF-8")))
 			rank = 1;
 		else if (sortKey.startsWith(QLatin1String("UTF-16")))
 			rank = 2;
-		else if (iso8859RegExp.exactMatch(sortKey)) {
-			if (iso8859RegExp.cap(1).size() == 1)
+		else if (iso8859Match.hasMatch()) {
+			if (iso8859Match.captured(1).size() == 1)
 				rank = 3;
 			else
 				rank = 4;
@@ -361,11 +362,11 @@ QList<QTextCodec*> *TWUtils::findCodecs()
 	return codecList;
 }
 
-QStringList* TWUtils::translationList = NULL;
+QStringList* TWUtils::translationList = nullptr;
 
 QStringList* TWUtils::getTranslationList()
 {
-	if (translationList != NULL)
+	if (translationList)
 		return translationList;
 
 	translationList = new QStringList;
@@ -394,82 +395,6 @@ QStringList* TWUtils::getTranslationList()
 	return translationList;
 }
 
-QHash<QString, QString>* TWUtils::dictionaryList = NULL;
-
-QHash<QString, QString>* TWUtils::getDictionaryList(const bool forceReload /* = false */)
-{
-	if (dictionaryList != NULL) {
-		if (!forceReload)
-			return dictionaryList;
-		delete dictionaryList;
-	}
-
-	dictionaryList = new QHash<QString, QString>();
-	foreach (QDir dicDir, TWUtils::getLibraryPath(QString::fromLatin1("dictionaries")).split(QLatin1String(PATH_LIST_SEP))) {
-		foreach (QFileInfo dicFileInfo, dicDir.entryInfoList(QStringList(QString::fromLatin1("*.dic")),
-					QDir::Files | QDir::Readable, QDir::Name | QDir::IgnoreCase)) {
-			QFileInfo affFileInfo(dicFileInfo.dir(), dicFileInfo.completeBaseName() + QLatin1String(".aff"));
-			if (affFileInfo.isReadable())
-				dictionaryList->insertMulti(dicFileInfo.canonicalFilePath(), dicFileInfo.completeBaseName());
-		}
-	}
-	
-	TWApp::instance()->notifyDictionaryListChanged();
-	return dictionaryList;
-}
-
-QHash<const QString,Hunhandle*> *TWUtils::dictionaries = NULL;
-
-Hunhandle* TWUtils::getDictionary(const QString& language)
-{
-	if (language.isEmpty())
-		return NULL;
-	
-	if (dictionaries == NULL)
-		dictionaries = new QHash<const QString,Hunhandle*>;
-	
-	if (dictionaries->contains(language))
-		return dictionaries->value(language);
-	
-	Hunhandle *h = NULL;
-	foreach (QDir dicDir, TWUtils::getLibraryPath(QString::fromLatin1("dictionaries")).split(QLatin1String(PATH_LIST_SEP))) {
-		QFileInfo affFile(dicDir, language + QLatin1String(".aff"));
-		QFileInfo dicFile(dicDir, language + QLatin1String(".dic"));
-		if (affFile.isReadable() && dicFile.isReadable()) {
-			h = Hunspell_create(affFile.canonicalFilePath().toLocal8Bit().data(),
-			                    dicFile.canonicalFilePath().toLocal8Bit().data());
-			(*dictionaries)[language] = h;
-			return h;
-		}
-	}
-	return NULL;
-}
-
-QString TWUtils::getLanguageForDictionary(const Hunhandle * pHunspell)
-{
-	if (!pHunspell || !dictionaries)
-		return QString();
-	
-	for (QHash<const QString,Hunhandle*>::const_iterator it = dictionaries->begin(); it != dictionaries->end(); ++it) {
-		if (it.value() == pHunspell)
-			return it.key();
-	}
-	return QString();
-}
-
-void TWUtils::clearDictionaries()
-{
-	if (!dictionaries)
-		return;
-	
-	for (QHash<const QString,Hunhandle*>::iterator it = dictionaries->begin(); it != dictionaries->end(); ++it) {
-		if (it.value())
-			Hunspell_destroy(it.value());
-	}
-	delete dictionaries;
-	dictionaries = NULL;
-}
-
 QStringList* TWUtils::filters;
 QStringList* TWUtils::filterList()
 {
@@ -498,8 +423,7 @@ QString TWUtils::chooseDefaultFilter(const QString & filename, const QStringList
 
 	if (extension.isEmpty())
 		return filters[0];
-	
-	QRegExp re(QString::fromLatin1("\\*\\.") + QRegExp::escape(extension));
+
 	foreach (QString filter, filters) {
 		// return filter if it corresponds to the given extension
 		// note that the extension must be the first one in the list to match;
@@ -550,7 +474,7 @@ QStringList TWUtils::constructUniqueFileLabels(const QStringList & fileList)
 
 void TWUtils::updateRecentFileActions(QObject *parent, QList<QAction*> &actions, QMenu *menu, QAction * clearAction) /* static */
 {
-	QSETTINGS_OBJECT(settings);
+	Tw::Settings settings;
 	QStringList fileList, labelList;
 	if (settings.contains(QString::fromLatin1("recentFiles"))) {
 		QList<QVariant> files = settings.value(QString::fromLatin1("recentFiles")).toList();
@@ -599,6 +523,9 @@ void TWUtils::updateRecentFileActions(QObject *parent, QList<QAction*> &actions,
 	}
 
 	for (int i = 0; i < numRecentFiles; ++i) {
+		// a "&" inside a menu label is considered a mnemonic, thus, we need to escape them
+		labelList[i].replace(QString::fromLatin1("&"), QString::fromLatin1("&&"));
+
 		actions[i]->setText(labelList[i]);
 		actions[i]->setData(fileList[i]);
 		actions[i]->setVisible(true);
@@ -622,9 +549,9 @@ void TWUtils::updateWindowMenu(QWidget *window, QMenu *menu) /* static */
 	while (!menu->actions().isEmpty() && menu->actions().last()->isSeparator())
 		menu->removeAction(menu->actions().last());
 	
-	QList<TeXDocument *> texDocList;
+	QList<TeXDocumentWindow *> texDocList;
 	QStringList fileList, labelList;
-	Q_FOREACH(TeXDocument * texDoc, TeXDocument::documentList()) {
+	Q_FOREACH(TeXDocumentWindow * texDoc, TeXDocumentWindow::documentList()) {
 		texDocList.append(texDoc);
 		fileList.append(texDoc->fileName());
 	}
@@ -633,7 +560,7 @@ void TWUtils::updateWindowMenu(QWidget *window, QMenu *menu) /* static */
 	// append an item for each TeXDocument
 	bool first = true;
 	for (int i = 0; i < texDocList.size(); ++i) {
-		TeXDocument * texDoc = texDocList[i];
+		TeXDocumentWindow * texDoc = texDocList[i];
 		if (first && !menu->actions().isEmpty())
 			menu->addSeparator();
 		first = false;
@@ -643,7 +570,7 @@ void TWUtils::updateWindowMenu(QWidget *window, QMenu *menu) /* static */
 			f.setItalic(true);
 			selWin->setFont(f);
 		}
-		if (texDoc == qobject_cast<TeXDocument*>(window)) {
+		if (texDoc == qobject_cast<TeXDocumentWindow*>(window)) {
 			selWin->setCheckable(true);
 			selWin->setChecked(true);
 		}
@@ -651,10 +578,10 @@ void TWUtils::updateWindowMenu(QWidget *window, QMenu *menu) /* static */
 		menu->addAction(selWin);
 	}
 
-	QList<PDFDocument *> pdfDocList;
+	QList<PDFDocumentWindow *> pdfDocList;
 	fileList.clear();
 	labelList.clear();
-	Q_FOREACH(PDFDocument * pdfDoc, PDFDocument::documentList()) {
+	Q_FOREACH(PDFDocumentWindow * pdfDoc, PDFDocumentWindow::documentList()) {
 		pdfDocList.append(pdfDoc);
 		fileList.append(pdfDoc->fileName());
 	}
@@ -663,12 +590,12 @@ void TWUtils::updateWindowMenu(QWidget *window, QMenu *menu) /* static */
 	// append an item for each PDFDocument
 	first = true;
 	for (int i = 0; i < pdfDocList.size(); ++i) {
-		PDFDocument * pdfDoc = pdfDocList[i];
+		PDFDocumentWindow * pdfDoc = pdfDocList[i];
 		if (first && !menu->actions().isEmpty())
 			menu->addSeparator();
 		first = false;
 		SelWinAction *selWin = new SelWinAction(menu, fileList[i], labelList[i]);
-		if (pdfDoc == qobject_cast<PDFDocument*>(window)) {
+		if (pdfDoc == qobject_cast<PDFDocumentWindow*>(window)) {
 			selWin->setCheckable(true);
 			selWin->setChecked(true);
 		}
@@ -736,7 +663,7 @@ void TWUtils::zoomToHalfScreen(QWidget *window, bool rhs)
 			// hacking the config files.
 			// (Note: this should only be necessary in some special cases, e.g.
 			// on X11 systems with special effects enabled)
-			QSETTINGS_OBJECT(settings);
+			Tw::Settings settings;
 			wDiff = qMax(0, settings.value(QString::fromLatin1("windowWDiff"), 0).toInt());
 			hDiff = qMax(0, settings.value(QString::fromLatin1("windowHDiff"), 0).toInt());
 		}
@@ -855,7 +782,7 @@ void TWUtils::applyToolbarOptions(QMainWindow *theWindow, int iconSize, bool sho
 	iconSize = iconSize * 8 + 8;	// convert 1,2,3 to 16,24,32
 	foreach (QObject *object, theWindow->children()) {
 		QToolBar *theToolBar = qobject_cast<QToolBar*>(object);
-		if (theToolBar != NULL) {
+		if (theToolBar) {
 			theToolBar->setToolButtonStyle(showText ? Qt::ToolButtonTextUnderIcon : Qt::ToolButtonIconOnly);
 			theToolBar->setIconSize(QSize(iconSize, iconSize));
 		}
@@ -885,7 +812,7 @@ bool TWUtils::findNextWord(const QString& text, int index, int& start, int& end)
 		return false;
 	QChar	ch = text.at(index);
 
-#define IS_WORD_FORMING(ch) (ch.isLetter() || ch.isMark())
+#define IS_WORD_FORMING(ch) ((ch).isLetter() || (ch).isMark())
 
 	if (IS_WORD_FORMING(ch) || ch == QChar::fromLatin1('@') /* || ch == QChar::fromLatin1('\'') || ch == 0x2019 */) {
 		bool isControlSeq = false; // becomes true if we include an @ sign or a leading backslash
@@ -1044,11 +971,11 @@ void TWUtils::readConfig()
 	pairClosers.clear();
 
 	QDir configDir(TWUtils::getLibraryPath(QString::fromLatin1("configuration")));
-	QRegExp pair(QString::fromLatin1("([^\\s])\\s+([^\\s])\\s*(?:#.*)?"));
+	QRegularExpression pair(QString::fromLatin1("^([^\\s])\\s+([^\\s])\\s*(?:#.*)?$"));
 
 	QFile pairsFile(configDir.filePath(QString::fromLatin1("delimiter-pairs.txt")));
 	if (pairsFile.open(QIODevice::ReadOnly)) {
-		while (1) {
+		while (true) {
 			QByteArray ba = pairsFile.readLine();
 			if (ba.size() == 0)
 				break;
@@ -1056,9 +983,10 @@ void TWUtils::readConfig()
 				continue;
 			ba.chop(1);
 			QString line = QString::fromUtf8(ba.data(), ba.size());
-			if (pair.exactMatch(line)) {
-				pairClosers[pair.cap(1).at(0)] = pair.cap(2).at(0);
-				pairOpeners[pair.cap(2).at(0)] = pair.cap(1).at(0);
+			QRegularExpressionMatch pairMatch = pair.match(line);
+			if (pairMatch.hasMatch()) {
+				pairClosers[pairMatch.captured(1).at(0)] = pairMatch.captured(2).at(0);
+				pairOpeners[pairMatch.captured(2).at(0)] = pairMatch.captured(1).at(0);
 			}
 		}
 	}
@@ -1075,19 +1003,20 @@ void TWUtils::readConfig()
 	
 	QFile configFile(configDir.filePath(QString::fromLatin1("texworks-config.txt")));
 	if (configFile.open(QIODevice::ReadOnly)) {
-		QRegExp keyVal(QString::fromLatin1("([-a-z]+):\\s*([^ \\t].+)"));
+		QRegularExpression keyVal(QStringLiteral("^([-a-z]+):\\s*([^ \\t].+)$"));
 			// looking for keyword, colon, optional whitespace, value
-		while (1) {
+		while (true) {
 			QByteArray ba = configFile.readLine();
 			if (ba.size() == 0)
 				break;
 			if (ba[0] == '#' || ba[0] == '\n')
 				continue;
 			QString	line = QString::fromUtf8(ba.data(), ba.size());
-			if (keyVal.exactMatch(line)) {
+			QRegularExpressionMatch keyValMatch = keyVal.match(line);
+			if (keyValMatch.hasMatch()) {
 				// if that matched, keyVal.cap(1) is the keyword, cap(2) is the value
-				const QString& keyword = keyVal.cap(1);
-				QString value = keyVal.cap(2).trimmed();
+				const QString& keyword = keyValMatch.captured(1);
+				QString value = keyValMatch.captured(2).trimmed();
 				if (keyword == QString::fromLatin1("include-text")) {
 					sIncludeTextCommand = value.replace(QString::fromLatin1("#RET#"), QChar::fromLatin1('\n'));
 					continue;
@@ -1156,11 +1085,11 @@ int TWUtils::findOpeningDelim(const QString& text, int pos)
 	return -1;
 }
 
-void TWUtils::installCustomShortcuts(QWidget * widget, bool recursive /* = true */, QSettings * map /* = NULL */)
+void TWUtils::installCustomShortcuts(QWidget * widget, bool recursive /* = true */, QSettings * map /* = nullptr */)
 {
 	bool deleteMap = false;
 	
-	if (widget == NULL)
+	if (!widget)
 		return;
 
 	if (!map) {
@@ -1217,8 +1146,6 @@ QDateTime TWUtils::gitCommitDate()
 	return QDateTime::fromString(QString::fromLatin1(GIT_COMMIT_DATE), Qt::ISODate).toUTC();
 }
 
-#pragma mark === SelWinAction ===
-
 // action subclass used for dynamic window-selection items in the Window menu
 
 SelWinAction::SelWinAction(QObject *parent, const QString &fileName, const QString &label)
@@ -1228,16 +1155,14 @@ SelWinAction::SelWinAction(QObject *parent, const QString &fileName, const QStri
 	setData(fileName);
 }
 
-#pragma mark === CmdKeyFilter ===
-
 // on OS X only, the singleton CmdKeyFilter object is attached to all TeXDocument editor widgets
 // to stop Command-keys getting inserted into edit text items
 
-CmdKeyFilter *CmdKeyFilter::filterObj = NULL;
+CmdKeyFilter *CmdKeyFilter::filterObj = nullptr;
 
 CmdKeyFilter *CmdKeyFilter::filter()
 {
-	if (filterObj == NULL)
+	if (!filterObj)
 		filterObj = new CmdKeyFilter;
 	return filterObj;
 }
@@ -1261,114 +1186,3 @@ bool CmdKeyFilter::eventFilter(QObject *obj, QEvent *event)
 	return QObject::eventFilter(obj, event);
 }
 
-/*static*/
-FileVersionDatabase FileVersionDatabase::load(const QString & path)
-{
-	QFile fin(path);
-	FileVersionDatabase retVal;
-	QDir rootDir(QFileInfo(path).absoluteDir());
-	
-	if (!fin.open(QIODevice::ReadOnly | QIODevice::Text))
-		return retVal;
-
-	QTextStream strm(&fin);
-	
-	while (!strm.atEnd()) {
-		FileVersionDatabase::Record rec;
-		QString line = strm.readLine().trimmed();
-		
-		// ignore comments
-		if (line.startsWith(QChar::fromLatin1('#'))) continue;
-		
-		rec.version = line.section(QChar::fromLatin1(' '), 0, 0);
-		rec.hash = QByteArray::fromHex(line.section(QChar::fromLatin1(' '), 1, 1).toLatin1());
-		rec.filePath = line.section(QChar::fromLatin1(' '), 2).trimmed();
-		rec.filePath = rootDir.absoluteFilePath(rec.filePath.filePath());
-		retVal.m_records.append(rec);
-	}
-	
-	fin.close();
-	
-	return retVal;
-}
-
-bool FileVersionDatabase::save(const QString & path) const
-{
-	QFile fout(path);
-	QDir rootDir(QFileInfo(path).absoluteDir());
-	
-	if (!fout.open(QIODevice::WriteOnly | QIODevice::Text))
-		return false;
-	
-	QTextStream strm(&fout);
-
-	foreach (FileVersionDatabase::Record rec, m_records) {
-		QString filePath = rec.filePath.absoluteFilePath();
-		strm << rec.version << " " << rec.hash.toHex() << " " << rootDir.relativeFilePath(filePath) << endl;
-	}
-	
-	fout.close();
-	return true;
-}
-
-void FileVersionDatabase::addFileRecord(const QFileInfo & file, const QByteArray & md5Hash, const QString version)
-{
-	// remove all existing entries for this file
-	QMutableListIterator<FileVersionDatabase::Record> it(m_records);
-	
-	while (it.hasNext()) {
-		const FileVersionDatabase::Record rec = it.next();
-		if (file.absoluteFilePath() == rec.filePath.absoluteFilePath()) {
-			it.remove();
-		}
-	}
-	
-	// add the new data
-	FileVersionDatabase::Record rec;
-	rec.filePath = file;
-	rec.version = version;
-	rec.hash = md5Hash;
-	m_records.append(rec);
-}
-
-bool FileVersionDatabase::hasFileRecord(const QFileInfo & file) const
-{
-	QListIterator<FileVersionDatabase::Record> it(m_records);
-	
-	while (it.hasNext()) {
-		const FileVersionDatabase::Record rec = it.next();
-		if (file.filePath() == rec.filePath.filePath())
-			return true;
-	}
-	return false;
-}
-
-FileVersionDatabase::Record FileVersionDatabase::getFileRecord(const QFileInfo & file) const
-{
-	QListIterator<FileVersionDatabase::Record> it(m_records);
-	
-	while (it.hasNext()) {
-		const FileVersionDatabase::Record rec = it.next();
-		if (file == rec.filePath)
-			return rec;
-	}
-
-	FileVersionDatabase::Record retVal;
-	retVal.version = QString();
-	retVal.hash = QByteArray::fromHex("d41d8cd98f00b204e9800998ecf8427e"); // hash for the zero-length string
-	return retVal;
-}
-
-/*static*/
-QByteArray FileVersionDatabase::hashForFile(const QString & path)
-{
-	QByteArray retVal = QByteArray::fromHex("d41d8cd98f00b204e9800998ecf8427e"); // hash for the zero-length string;
-	QFile fin(path);
-	
-	if (!fin.open(QIODevice::ReadOnly))
-		return retVal;
-	
-	retVal = QCryptographicHash::hash(fin.readAll(), QCryptographicHash::Md5);
-	fin.close();
-	return retVal;
-}
